<?php

namespace Drupal\backstop_generator;

/**
 * A service definiton for retreiving backstop config.
 */
class BackstopConfigRetrieverService {
  protected $configurationObject;

  /**
   * Create config.
   */
  public function __construct() {
    $config = \Drupal::config('backstop_generator.backstopconfiguration');

    $this->configurationObject = [];
    $this->configurationObject['id'] = strtolower(str_replace(' ', '_', \Drupal::config('system.site')->get('name')));
    $this->configurationObject['viewports'] = [];

    if ($config->get('viewports.iphone_h')) {
      $viewport = [
        'label' => 'iPhone_h',
        'width' => 667,
        'height' => 375,
        'deviceScaleFactor' => 2.0,
        'pageScaleFactor' => 1.0,
        'mobile' => true,
        'userAgent' => 'Mozilla/5.0 (iPhone; CPU iPhone OS 11_0 like Mac OS X) AppleWebKit/604.1.38 (KHTML, like Gecko) Version/11.0 Mobile/15A372 Safari/604.1'
      ];
      array_push($this->configurationObject['viewports'], $viewport);
    }

    if ($config->get('viewports.iphone_v')) {
      $viewport = [
        'label' => 'iPhone_v',
        'width' => 375,
        'height' => 667,
        'deviceScaleFactor' => 2.0,
        'pageScaleFactor' => 1.0,
        'mobile' => true,
        'userAgent' => "Mozilla/5.0 (iPhone; CPU iPhone OS 11_0 like Mac OS X) AppleWebKit/604.1.38 (KHTML, like Gecko) Version/11.0 Mobile/15A372 Safari/604.1"
      ];
      array_push($this->configurationObject['viewports'], $viewport);
    }
    if ($config->get('viewports.tablet_v')) {
      $viewport = [
        'label' => 'Tablet Vertical',
        'width' => 768,
        'height' => 1024,
      ];
      array_push($this->configurationObject['viewports'], $viewport);
    }
    if ($config->get('viewports.Tablet')) {
      $viewport = [
        'label' => 'Tablet',
        'width' => 1024,
        'height' => 768,
      ];
      array_push($this->configurationObject['viewports'], $viewport);
    }
    if ($config->get('viewports.Desktop')) {
      $viewport = [
        'label' => 'Desktop',
        'width' => 1400,
        'height' => 900,
      ];
      array_push($this->configurationObject['viewports'], $viewport);
    }
    if ($config->get('viewports.HD')) {
      $viewport = [
        'label' => 'HD',
        'width' => 1920,
        'height' => 1080,
      ];
      array_push($this->configurationObject['viewports'], $viewport);
    }
    if ($config->get('viewports.HD+')) {
      $viewport = [
        'label' => 'HD+',
        'width' => 2560,
        'height' => 1440,
      ];
      array_push($this->configurationObject['viewports'], $viewport);
    }
    if ($config->get('viewports.UHD')) {
      $viewport = [
        'label' => 'UHD',
        'width' => 3840,
        'height' => 2160,
      ];
      array_push($this->configurationObject['viewports'], $viewport);
    }
    $this->configurationObject['onBeforeScript'] = "puppeteer/onBefore.js";
    $this->configurationObject['onReadyScript'] = "puppeteer/onReady.js";
    $this->configurationObject['scenarios'] = [];
    $this->configurationObject['paths'] = [
      'bitmaps_reference' => 'backstop_data/bitmaps_reference',
      "bitmaps_test" => "backstop_data/bitmaps_test",
      "engine_scripts" => "backstop_data/engine_scripts",
      "html_report" => "backstop_data/html_report",
      "ci_report" => "backstop_data/ci_report",
    ];
    $this->configurationObject['report'] = ['browser'];
    $this->configurationObject['engine'] = "puppeteer";
    $this->configurationObject['engineFlags'] = [];
    $this->configurationObject['asyncCaptureLimit'] = 5;
    $this->configurationObject['asyncCompareLimit'] = 50;
    $this->configurationObject['debug'] = FALSE;
    $this->configurationObject['debugWindow'] = FALSE;

    $pages_config_array = $config->get('pre_defined_pages_table');
    $randomExclude = [];
    foreach ($pages_config_array as $key => $value) {
      if ($value['page'] != NULL) {
        $node = \Drupal::entityTypeManager()->getStorage('node')->load($value['page']);
        $url = $node->toUrl('canonical', ['absolute' => TRUE])->toString();
        $referenceUrl = '';
        if ($config->get('referenceurl')) {
          $referenceUrl = $config->get('referenceurl') . '/node/' . $node->id();
        }
        $scenario = [
          'label' => $node->getTitle(),
          'cookiePath' => "backstop_data/engine_scripts/cookies.json",
          'url' => $url,
          "referenceUrl" => $referenceUrl,
          "readyEvent" => "",
          "readySelector" => "",
          "delay" => 0,
          "hideSelectors" => [],
          "removeSelectors" => [],
          "hoverSelector" => "",
          "clickSelector" => "",
          "postInteractionWait" => "",
          "selectors" => [],
          "selectorExpansion" => TRUE,
          "misMatchThreshold" => 0.1,
          "requireSameDimensions" => TRUE,
        ];
        array_push($this->configurationObject['scenarios'], $scenario);
        array_push($randomExclude, $value['page']);
      }
    }
    $randomCount = $config->get('additional_random_pages');
    if (!empty($randomExclude)) {
      $nids = \Drupal::entityQuery('node')
        ->condition('nid', $randomExclude, 'NOT IN')
        ->addTag('random')
        ->range(0, $randomCount)
        ->execute();
    }
    if ($nids) {
      foreach ($nids as $nid) {
        $node = \Drupal::entityTypeManager()->getStorage('node')->load($nid);
        $url = $node->toUrl('canonical', ['absolute' => TRUE])->toString();
        $referenceUrl = '';
        if ($config->get('referenceurl')) {
          $referenceUrl = $config->get('referenceurl') . '/node/' . $nid;
        }
        $scenario = [
          'label' => $node->getTitle(),
          'cookiePath' => "backstop_data/engine_scripts/cookies.json",
          'url' => $url,
          "referenceUrl" => $referenceUrl,
          "readyEvent" => "",
          "readySelector" => "",
          "delay" => 0,
          "hideSelectors" => [],
          "removeSelectors" => [],
          "hoverSelector" => "",
          "clickSelector" => "",
          "postInteractionWait" => "",
          "selectors" => [],
          "selectorExpansion" => TRUE,
          "misMatchThreshold" => 0.1,
          "requireSameDimensions" => TRUE,
        ];
        array_push($this->configurationObject['scenarios'], $scenario);
      }
    }
  }

  /**
   * Expose method for calling constructed config.
   */
  public function getConfigurationObject() {
    return $this->configurationObject;
  }

}
